#pragma once

#include "GraphicsAPI.h"


class T2dBound {
	private:
		D3DXVECTOR3 fMin, fMax;

		#define UpdateRange(_pt_, _axis_) {				\
			if (_pt_.##_axis_## > fMax.##_axis_##)		\
				fMax.##_axis_## = _pt_.##_axis_##;		\
			if (_pt_.##_axis_## < fMin.##_axis_##)		\
					fMin.##_axis_## = _pt_.##_axis_##;	\
		}

		#define PtInRange(_pt_, _obj_, _axis_) 				\
					((_pt_.##_axis_## >= ##_obj_##.fMin.##_axis_##) && (_pt_.##_axis_## <= ##_obj_##.fMax.##_axis_##))
		
	public:

		T2dBound() {
			// this is no a valid bound
			ResetBound();
		}

		T2dBound(float xMin, float xMax, float yMin, float yMax) {
			SetBound(xMin, xMax, yMin, yMax);
		}

		float Left() { return fMin.x; }
		float Right() { return fMax.x; }
		float Bottom() { return fMin.y; }
		float Top() { return fMax.y; }

		const D3DXVECTOR3& MinPosition() const { return fMin;}
		const D3DXVECTOR3& MaxPosition() const { return fMax;}

		float Width() const { return fMax.x - fMin.x; }
		float Height() const { return fMax.y - fMin.y; }

		void SetBound(float xMin, float xMax, float yMin, float yMax) {
			fMin.x = xMin;
			fMax.x = xMax;
			fMin.y = yMin;
			fMax.y = yMax;
		}

			//
			// give a new width and height without
			// moving the bound ...
		void SetBoundSizeTo(float w, float h) {
			fMax.x = fMin.x + w;
			fMax.y = fMin.y + h;
		}

			//
			// Move the entire bound by (dx,dy)
		void MoveBoundBy(float dx, float dy) {
			fMin.x += dx;
			fMax.x += dx;
			fMin.y += dy;
			fMax.y += dy;
		}

			//
			// Move the min pt to (x,y) maintining w/h
			// of the bound
		void SetBoundMinTo(float x, float y) {
			float w = Width();
			float h = Height();
			fMin.x = x;
			fMax.x = x + w;
			fMin.y = y;
			fMax.y = y + h;
		}

		void ResetBound() {
			// this is no a valid bound
			fMin.x = fMin.y =  100000000;
			fMax.x = fMax.y = -100000000;
		}

		bool ValidBound() const {
			return (fMax.x > fMin.x) && (fMax.y > fMin.y);
		}

		void UpdateBound(const D3DXVECTOR3 pt) {
			UpdateRange(pt, x);
			UpdateRange(pt, y);
		}

		void UpdateBound(const T2dBound &b) {
			UpdateBound(b.fMin);
			UpdateBound(b.fMax);
		}

		bool BoundIntersect(const T2dBound &b) const {
			return (
					  PtInRange(b.fMin, (*this), x) || 
			          PtInRange(b.fMax, (*this), x) ||
					  PtInRange(fMin, b, x) ||
					  PtInRange(fMax, b, x)
				   ) && 
				   (
					  PtInRange(b.fMin, (*this), y) || 
			          PtInRange(b.fMax, (*this), y) ||
					  PtInRange(fMin, b, y) ||
					  PtInRange(fMax, b, y)
				   );
		}

		bool PointInBound(const D3DXVECTOR3 pt) const {
			return ( PtInRange(pt, (*this), x) && PtInRange(pt, (*this), y) );
		}

		void GetBoundaryPoints(D3DXVECTOR3 pt[4]) const {
			pt[0].x = fMin.x;
			pt[0].y = fMax.y;

			pt[1].x = fMax.x;
			pt[1].y = fMax.y;

			pt[2].x = fMax.x;
			pt[2].y = fMin.y;

			pt[3].x = fMin.x;
			pt[3].y = fMin.y;
		}

};
